==> litejs-0.9.1.tgz: package/model.js <==



/*
* @version    0.3.1
* @date       2015-12-22
* @stability  1 - Experimental
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



!function(root) {
	var arrProto = Array.prototype

	function Model(data) {
		var model = this
		model.data = data
		model.lists = []
		if (model.init) {
			model.init(data)
		}
		model.set(data)
	}

	Object.merge(Model.prototype, Event.Emitter, {
		set: function(key, val) {
			var model = this
			, changed = []
			, data = key

			if (typeof key != "object") {
				JSON.pointer(data = {}, key, val)
			}

			if (model.validate(data)) {
				JSON.mergePatch(model.data, data, changed)
			}

			if (changed.length) {
				model.emit("change", changed, model)
				changed.map(function(pointer) {
					model.emit("change:" + pointer, JSON.pointer(model.data, pointer))
				})
				model.lists.map(function(list) {
					list.emit("change", changed, model)
				})
			}
			return changed
		},
		get: function(name, fallback) {
			var val
			if (name && name.charAt(0) == "/") {
				val = JSON.pointer(this.data, name)
			} else {
				val = this.data[name]
			}
			return val == void 0 && fallback || val
		},
		format: function(format, scope) {
			format = (format || "").replace(/{\s*([\w$]*)#([\w\/]+)/g, function(_, $1, $2) {
				return "{" + ($1 || "this") + "|get:'" + $2 + "'"
			})
			return format.format(this, this.data, scope || {})
		},
		validate: returnTrue,
		destroy: function(next) {
			var model = this
			, arr = model.lists.slice()
			, len = arr.length

			for (; len--; ) {
				arr[len].remove(model)
			}

			model.emit("destroy")
			model.off()
			//delete model.constructor.cached[model.get("id")]
		},
		then: _then,
		toJSON: function() {
			return this.data
		}
	})

	root.Model = Model.cache(true, function(data) {
		return data.id
	})

	function List(name, _a1, _a2) {
		var list = this
		list.name = name
		if (list.init) {
			list.init(name, _a1, _a2)
		}
	}

	Object.merge(List.prototype = [], Event.Emitter, {
		wait: Fn.hold,
		syncMethods: [
			"add", "emit", "filterFn", "merge",
			"model", "pluck", "remove", "removeAll", "sortFn"
		],
		model: root.Model,
		add: function(item) {
			var pos
			, list = this

			if (item) {
				if (!(item instanceof Model)) {
					item = list.model(item)
				}

				if (item.lists.indexOf(list) == -1) {
					pos = indexFor(list, item, list.sortFn)
					if (list.filterFn(item, pos)) {
						item.lists.push(list)
						list.splice(pos , 0, item)
						list.emit("add", item, pos)
					}
				}
			}
			return list
		},
		filterFn: returnTrue,
		remove: function(item) {
			var list = this
			if (item && item.lists && item.lists.remove(list) > -1) {
				list.emit("remove", item, arrProto.remove.call(list, item))
			}
			return list
		},
		removeAll: function(){
			for (var list = this, len = list.length; len--; ) {
				list.remove(list[len])
			}
			return list
		},
		each: function(fn, scope) {
			arrProto.each.call(this, fn, scope)
			return this
		},
		get: function(id, next, scope) {
			var list = this
			, item = list.model.cached[id]

			if (item && item.lists.indexOf(list) != -1) {
				return next ?
					(next.call(scope || list, null, item), list) :
					item
			}

			next && next.call(scope || list, Error("Item not found"))
		},
		at: function(index, next, scope) {
			var list = this
			, item = list[index]
			, err = item ? null : Error("Item not found")

			return next ?
				(next.call(scope || list, err, item), list) :
				item
		},
		first: function(next, scope) {
			return this.at(0, next, scope)
		},
		last: function(next, scope) {
			return this.at(this.length - 1, next, scope)
		},
		then: _then,
		onEmpty: function(next, scope) {
			if (!this.length) next.call(scope || this)
			return this
		},
		pluck: function(field, next, scope) {
			var list = this
			, arr = arrProto.map.call(list, function(item) {
				return item.get(field)
			})
			return next ?
				(next.call(scope || list, arr), list) :
				arr
		},
		set: function(key, val) {
			this.each(function(item) {
				item.set(key, val)
			})
			return this
		},
		merge: function(mergeList) {
			var list = this
			, lists = list.lists || (list.lists = [])

			if (mergeList && lists.indexOf(mergeList) == -1) {
				lists.push(mergeList)

				mergeList
				.each(list.add, list)
				.on("add", list.add, list)
				.on("remove", _remove, list)
				.on("change", _change, list)
				.then(list.wait())
			}

			return list
		},
		paged: function(num) {
			var list = this
			, view = new List()
			, start = 0
			, end = num

			view.filterFn = function(item, pos) {
				pos = list.indexOf(item)
				return pos >= start && pos < end
			}

			view.pages = new List()
			view.setPage = function(newPage) {
				var lastPage = 0 | (list.length / num)
				if (newPage > lastPage) {
					newPage = lastPage
				}
				view.page = newPage
				start = num * newPage
				end = start + num
				view.reFilter()
			}

			view.merge(list)

			return view
		},
		reFilter: function() {
			var list = this
			, lists = list.lists || (list.lists = [])

			lists.each(function(parent) {
				parent.each(function(item) {
					_change.call(list, null, item)
				})
			})
		},
		reSort: function(fn) {
			var item
			, list = this
			, len = list.length
			, removed = []

			if (typeof fn == "string") {
				fn = [fn]
			}

			if (Array.isArray(fn)) {
				fn = Fn("a b->" + fn.map(function(key) { return "" }).join("||"))
			}

			if (fn) {
				list.sortFn = fn
			}

			for (; --len; ) {
				item = list[len]
				if (fn(list[len - 1], item) > -1) {
					removed.push(item)
					list.remove(item)
				}
			}
			removed.each(list.add, list)
		},

		toJSON: function() {
			return this.slice(0)
		}
	})

	function _remove(item) {
		if (item.lists.every(notIn, this.lists)) {
			this.remove(item)
		}
	}

	function _change(changed, item) {
		var list = this
		, matches = list.filterFn(item)
		, inList = item.lists.indexOf(list) > -1

		if (matches && !inList) {
			list.add(item)
		}
		if (!matches && inList) {
			list.remove(item)
		}
	}


	root.List = List.cache(true, function(name) {
		return name
	}, {})



	function returnTrue() {
		return true
	}

	function _then(next, scope) {
		if (typeof next == "function") {
			next.call(scope || this)
		}
		return this
	}

	function notIn(v) {
		return this.indexOf(v) == -1
	}

	function indexFor(arr, el, compFn) {
		var o
		, i = 0
		, l = arr.length

		// Fast lookup for last position
		if (compFn && l && compFn(el, arr[l-1]) < 1) {
			// Otherwise binary search
			for (; i<l; ) {
				compFn(el, arr[o=(i+l)>>1]) < 0 ? l=o : i=o+1
			}
		}
		return l
	}

}(this)

==> litejs-0.9.2.tgz: package/model.js <==



/*
* @version    0.3.1
* @date       2015-12-22
* @stability  1 - Experimental
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



!function(root) {
	var arrProto = Array.prototype

	function Model(data) {
		var model = this
		model.data = data
		model.lists = []
		if (model.init) {
			model.init(data)
		}
		model.set(data)
	}

	JSON.merge(Model.prototype, Event.Emitter, {
		set: function(key, val) {
			var model = this
			, changed = []
			, data = key

			if (typeof key != "object") {
				JSON.pointer(data = {}, key, val)
			}

			if (model.validate(data)) {
				JSON.mergePatch(model.data, data, changed)
			}

			if (changed.length) {
				model.emit("change", changed, model)
				changed.map(function(pointer) {
					model.emit("change:" + pointer, JSON.pointer(model.data, pointer))
				})
				model.lists.map(function(list) {
					list.emit("change", changed, model)
				})
			}
			return changed
		},
		get: function(name, fallback) {
			var val
			if (name && name.charAt(0) == "/") {
				val = JSON.pointer(this.data, name)
			} else {
				val = this.data[name]
			}
			return val == void 0 && fallback || val
		},
		format: function(format, scope) {
			format = (format || "").replace(/{\s*([\w$]*)#([\w\/]+)/g, function(_, $1, $2) {
				return "{" + ($1 || "this") + "|get:'" + $2 + "'"
			})
			return format.format(this, this.data, scope || {})
		},
		validate: returnTrue,
		destroy: function(next) {
			var model = this
			, arr = model.lists.slice()
			, len = arr.length

			for (; len--; ) {
				arr[len].remove(model)
			}

			model.emit("destroy")
			model.off()
			//delete model.constructor.cached[model.get("id")]
		},
		then: _then,
		toJSON: function() {
			return this.data
		}
	})

	root.Model = Model.cache(true, function(data) {
		return data.id
	})

	function List(name, _a1, _a2) {
		var list = this
		list.name = name
		if (list.init) {
			list.init(name, _a1, _a2)
		}
	}

	JSON.merge(List.prototype = [], Event.Emitter, {
		wait: Fn.hold,
		syncMethods: [
			"add", "emit", "filterFn", "merge",
			"model", "pluck", "remove", "removeAll", "sortFn"
		],
		model: root.Model,
		add: function(item) {
			var pos
			, list = this

			if (item) {
				if (!(item instanceof Model)) {
					item = list.model(item)
				}

				if (item.lists.indexOf(list) == -1) {
					pos = indexFor(list, item, list.sortFn)
					if (list.filterFn(item, pos)) {
						item.lists.push(list)
						list.splice(pos , 0, item)
						list.emit("add", item, pos)
					}
				}
			}
			return list
		},
		filterFn: returnTrue,
		remove: function(item) {
			var list = this
			if (item && item.lists && item.lists.remove(list) > -1) {
				list.emit("remove", item, arrProto.remove.call(list, item))
			}
			return list
		},
		removeAll: function(){
			for (var list = this, len = list.length; len--; ) {
				list.remove(list[len])
			}
			return list
		},
		each: function(fn, scope) {
			arrProto.each.call(this, fn, scope)
			return this
		},
		get: function(id, next, scope) {
			var list = this
			, item = list.model.cached[id]

			if (item && item.lists.indexOf(list) != -1) {
				return next ?
					(next.call(scope || list, null, item), list) :
					item
			}

			next && next.call(scope || list, Error("Item not found"))
		},
		at: function(index, next, scope) {
			var list = this
			, item = list[index]
			, err = item ? null : Error("Item not found")

			return next ?
				(next.call(scope || list, err, item), list) :
				item
		},
		first: function(next, scope) {
			return this.at(0, next, scope)
		},
		last: function(next, scope) {
			return this.at(this.length - 1, next, scope)
		},
		then: _then,
		onEmpty: function(next, scope) {
			if (!this.length) next.call(scope || this)
			return this
		},
		pluck: function(field, next, scope) {
			var list = this
			, arr = arrProto.map.call(list, function(item) {
				return item.get(field)
			})
			return next ?
				(next.call(scope || list, arr), list) :
				arr
		},
		set: function(key, val) {
			this.each(function(item) {
				item.set(key, val)
			})
			return this
		},
		merge: function(mergeList) {
			var list = this
			, lists = list.lists || (list.lists = [])

			if (mergeList && lists.indexOf(mergeList) == -1) {
				lists.push(mergeList)

				mergeList
				.each(list.add, list)
				.on("add", list.add, list)
				.on("remove", _remove, list)
				.on("change", _change, list)
				.then(list.wait())
			}

			return list
		},
		paged: function(num) {
			var list = this
			, view = new List()
			, start = 0
			, end = num

			view.filterFn = function(item, pos) {
				pos = list.indexOf(item)
				return pos >= start && pos < end
			}

			view.pages = new List()
			view.setPage = function(newPage) {
				var lastPage = 0 | (list.length / num)
				if (newPage > lastPage) {
					newPage = lastPage
				}
				view.page = newPage
				start = num * newPage
				end = start + num
				view.reFilter()
			}

			view.merge(list)

			return view
		},
		reFilter: function() {
			var list = this
			, lists = list.lists || (list.lists = [])

			lists.each(function(parent) {
				parent.each(function(item) {
					_change.call(list, null, item)
				})
			})
		},
		reSort: function(fn) {
			var item
			, list = this
			, len = list.length
			, removed = []

			if (typeof fn == "string") {
				fn = [fn]
			}

			if (Array.isArray(fn)) {
				fn = Fn("a b->" + fn.map(function(key) { return "" }).join("||"))
			}

			fn = list.sortFn = fn || list.sortFn

			for (; --len; ) {
				item = list[len]
				if (fn(list[len - 1], item) > -1) {
					removed.push(item)
					list.remove(item)
				}
			}
			removed.each(list.add, list)
		},

		toJSON: function() {
			return this.slice(0)
		}
	})

	function _remove(item) {
		if (item.lists.every(notIn, this.lists)) {
			this.remove(item)
		}
	}

	function _change(changed, item) {
		var list = this
		, matches = list.filterFn(item)
		, inList = item.lists.indexOf(list) > -1

		if (matches && !inList) {
			list.add(item)
		}
		if (!matches && inList) {
			list.remove(item)
		}
	}


	root.List = List.cache(true, function(name) {
		return name
	}, {})



	function returnTrue() {
		return true
	}

	function _then(next, scope) {
		if (typeof next == "function") {
			next.call(scope || this)
		}
		return this
	}

	function notIn(v) {
		return this.indexOf(v) == -1
	}

	function indexFor(arr, el, compFn) {
		var o
		, i = 0
		, l = arr.length

		// Fast lookup for last position
		if (compFn && l && compFn(el, arr[l-1]) < 1) {
			// Otherwise binary search
			for (; i<l; ) {
				compFn(el, arr[o=(i+l)>>1]) < 0 ? l=o : i=o+1
			}
		}
		return l
	}

}(this)
